// tslint:disable max-classes-per-file
import { $callback } from "../symbols";
let nextLocalCallbackID = 1;
let nextRemoteCallbackID = 1;
const registeredLocalCallbacks = new Map();
const registeredRemoteCallbacks = new Map();
export function isCallback(thing) {
    return typeof thing === "function" && thing[$callback];
}
export function lookupLocalCallback(id) {
    return registeredLocalCallbacks.get(id);
}
function registerCallback(callback) {
    registeredLocalCallbacks.set(callback.id, callback);
    return callback;
}
function unregisterCallback(callback) {
    registeredLocalCallbacks.delete(callback.id);
    return callback;
}
export function lookupRemoteCallback(id) {
    return registeredRemoteCallbacks.get(id);
}
function registerRemoteCallback(callback) {
    registeredRemoteCallbacks.set(callback.id, callback);
    return callback;
}
function unregisterRemoteCallback(callback) {
    registeredRemoteCallbacks.delete(callback.id);
    return callback;
}
export function Callback(fn) {
    const callback = ((...args) => fn(...args));
    callback[$callback] = true;
    callback.id = nextLocalCallbackID++;
    callback.release = () => unregisterCallback(callback);
    return registerCallback(callback);
}
export function RemoteCallback(fn) {
    const callback = ((...args) => fn(...args));
    callback[$callback] = true;
    callback.id = nextRemoteCallbackID++;
    callback.release = () => unregisterRemoteCallback(callback);
    return registerRemoteCallback(callback);
}
export function SingleExposedCallback(fn) {
    const callback = ((...args) => fn(...args));
    callback[$callback] = true;
    callback.id = 0;
    callback.release = () => unregisterCallback(callback);
    return registerCallback(callback);
}
