import { isCallback } from "../common/callbacks";
import { isSerializedCallback, DefaultCallbackSerializer } from "./callbacks";
import { isSerializedError, DefaultErrorSerializer } from "./errors";
import { isIterator, isSerializedIterator, DefaultIteratorSerializer } from "./iterators";
export function extendSerializer(extend, implementation) {
    const fallbackSerializer = extend.serialize.bind(extend);
    return {
        deserialize(message, origin) {
            const fallback = (msg) => extend.deserialize(msg, origin);
            return implementation.deserialize(message, fallback);
        },
        serialize(input) {
            return implementation.serialize(input, fallbackSerializer);
        }
    };
}
export const DefaultSerializer = () => {
    const serializer = {
        deserialize(message, sender) {
            if (isSerializedError(message)) {
                return errorSerializer.deserialize(message, sender);
            }
            else if (isSerializedCallback(message)) {
                return callbackSerializer.deserialize(message, sender);
            }
            else if (isSerializedIterator(message)) {
                return iteratorSerializer.deserialize(message, sender);
            }
            else {
                return message;
            }
        },
        serialize(input) {
            if (input instanceof Error) {
                return errorSerializer.serialize(input);
            }
            else if (isCallback(input)) {
                return callbackSerializer.serialize(input);
            }
            else if (isIterator(input)) {
                return iteratorSerializer.serialize(input);
            }
            else {
                return input;
            }
        }
    };
    const callbackSerializer = DefaultCallbackSerializer(serializer);
    const errorSerializer = DefaultErrorSerializer();
    const iteratorSerializer = DefaultIteratorSerializer(serializer);
    return serializer;
};
