var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import DebugLogger from "debug";
import { createProxyFunction } from "../common/call-proxy";
import { Callback, RemoteCallback } from "../common/callbacks";
const debug = DebugLogger("threads:callback:messages");
export const DefaultIteratorSerializer = (rootSerializer) => ({
    deserialize(message, origin) {
        const remoteNext = createProxyFunction(origin, rootSerializer, message.next_fid, debug);
        const remoteCallback = RemoteCallback(remoteNext);
        const next = () => __awaiter(this, void 0, void 0, function* () {
            const result = yield remoteCallback();
            if (result.done) {
                remoteCallback.release();
            }
            return result;
        });
        const asyncIterator = {
            [Symbol.asyncIterator]: () => asyncIterator,
            next
        };
        return asyncIterator;
    },
    serialize(iter) {
        const next = Callback(() => __awaiter(this, void 0, void 0, function* () {
            const result = yield iter.next();
            if (result.done) {
                next.release();
            }
            return result;
        }));
        return {
            __iterator_marker: "$$iterator",
            next_fid: next.id
        };
    }
});
export const isIterator = (thing) => thing && typeof thing === "object" && (typeof thing.next === "function" ||
    typeof thing[Symbol.asyncIterator] === "function");
export const isSerializedIterator = (thing) => thing && typeof thing === "object" && "__iterator_marker" in thing && thing.__iterator_marker === "$$iterator";
