/**
 * Make sure that there is only ever one listener set on that event emitter.
 * Do so by setting a single event handler that then calls all the
 * event listeners.
 */
export function multiplexEventTarget(emitter) {
    const eventListeners = new Map();
    function addEventListener(event, listener) {
        if (eventListeners.has(event)) {
            eventListeners.get(event).add(listener);
        }
        else {
            eventListeners.set(event, new Set([listener]));
            emitter.addEventListener(event, (...args) => {
                const listeners = eventListeners.get(event) || [];
                for (const callback of listeners) {
                    callback(...args);
                }
            });
        }
    }
    function removeEventListener(event, listener) {
        if (eventListeners.has(event)) {
            eventListeners.get(event).delete(listener);
        }
    }
    return {
        addEventListener,
        removeEventListener
    };
}
