"use strict";
// tslint:disable max-classes-per-file
Object.defineProperty(exports, "__esModule", { value: true });
exports.SingleExposedCallback = exports.RemoteCallback = exports.Callback = exports.lookupRemoteCallback = exports.lookupLocalCallback = exports.isCallback = void 0;
const symbols_1 = require("../symbols");
let nextLocalCallbackID = 1;
let nextRemoteCallbackID = 1;
const registeredLocalCallbacks = new Map();
const registeredRemoteCallbacks = new Map();
function isCallback(thing) {
    return typeof thing === "function" && thing[symbols_1.$callback];
}
exports.isCallback = isCallback;
function lookupLocalCallback(id) {
    return registeredLocalCallbacks.get(id);
}
exports.lookupLocalCallback = lookupLocalCallback;
function registerCallback(callback) {
    registeredLocalCallbacks.set(callback.id, callback);
    return callback;
}
function unregisterCallback(callback) {
    registeredLocalCallbacks.delete(callback.id);
    return callback;
}
function lookupRemoteCallback(id) {
    return registeredRemoteCallbacks.get(id);
}
exports.lookupRemoteCallback = lookupRemoteCallback;
function registerRemoteCallback(callback) {
    registeredRemoteCallbacks.set(callback.id, callback);
    return callback;
}
function unregisterRemoteCallback(callback) {
    registeredRemoteCallbacks.delete(callback.id);
    return callback;
}
function Callback(fn) {
    const callback = ((...args) => fn(...args));
    callback[symbols_1.$callback] = true;
    callback.id = nextLocalCallbackID++;
    callback.release = () => unregisterCallback(callback);
    return registerCallback(callback);
}
exports.Callback = Callback;
function RemoteCallback(fn) {
    const callback = ((...args) => fn(...args));
    callback[symbols_1.$callback] = true;
    callback.id = nextRemoteCallbackID++;
    callback.release = () => unregisterRemoteCallback(callback);
    return registerRemoteCallback(callback);
}
exports.RemoteCallback = RemoteCallback;
function SingleExposedCallback(fn) {
    const callback = ((...args) => fn(...args));
    callback[symbols_1.$callback] = true;
    callback.id = 0;
    callback.release = () => unregisterCallback(callback);
    return registerCallback(callback);
}
exports.SingleExposedCallback = SingleExposedCallback;
