"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefaultSerializer = exports.extendSerializer = void 0;
const callbacks_1 = require("../common/callbacks");
const callbacks_2 = require("./callbacks");
const errors_1 = require("./errors");
const iterators_1 = require("./iterators");
function extendSerializer(extend, implementation) {
    const fallbackSerializer = extend.serialize.bind(extend);
    return {
        deserialize(message, origin) {
            const fallback = (msg) => extend.deserialize(msg, origin);
            return implementation.deserialize(message, fallback);
        },
        serialize(input) {
            return implementation.serialize(input, fallbackSerializer);
        }
    };
}
exports.extendSerializer = extendSerializer;
exports.DefaultSerializer = () => {
    const serializer = {
        deserialize(message, sender) {
            if (errors_1.isSerializedError(message)) {
                return errorSerializer.deserialize(message, sender);
            }
            else if (callbacks_2.isSerializedCallback(message)) {
                return callbackSerializer.deserialize(message, sender);
            }
            else if (iterators_1.isSerializedIterator(message)) {
                return iteratorSerializer.deserialize(message, sender);
            }
            else {
                return message;
            }
        },
        serialize(input) {
            if (input instanceof Error) {
                return errorSerializer.serialize(input);
            }
            else if (callbacks_1.isCallback(input)) {
                return callbackSerializer.serialize(input);
            }
            else if (iterators_1.isIterator(input)) {
                return iteratorSerializer.serialize(input);
            }
            else {
                return input;
            }
        }
    };
    const callbackSerializer = callbacks_2.DefaultCallbackSerializer(serializer);
    const errorSerializer = errors_1.DefaultErrorSerializer();
    const iteratorSerializer = iterators_1.DefaultIteratorSerializer(serializer);
    return serializer;
};
