import { SerializedError } from "./serializers";
export declare enum CommonMessageType {
    cancel = "call:cancel",
    error = "call:error",
    invoke = "call:invoke",
    result = "call:result",
    running = "call:running"
}
export declare type CallCancelMessage = {
    type: CommonMessageType.cancel;
    uid: number;
};
export declare type CallErrorMessage = {
    type: CommonMessageType.error;
    uid: number;
    error: SerializedError;
};
export declare type CallInvocationMessage = {
    type: CommonMessageType.invoke;
    /** Function ID */
    fid: number;
    /** Unique call ID */
    uid: number;
    args: any[];
};
export declare type CallResultMessage = {
    type: CommonMessageType.result;
    uid: number;
    complete?: true;
    payload?: any;
};
export declare type CallRunningMessage = {
    type: CommonMessageType.running;
    uid: number;
    resultType: "observable" | "promise";
};
export declare enum WorkerMessageType {
    init = "init",
    uncaughtError = "uncaughtError"
}
export declare type WorkerUncaughtErrorMessage = {
    type: WorkerMessageType.uncaughtError;
    error: {
        message: string;
        name: string;
        stack?: string;
    };
};
export declare type WorkerInitMessage = {
    type: WorkerMessageType.init;
    exposed: {
        type: "function";
    } | {
        type: "module";
        methods: Record<string, number>;
    };
};
